
# q-expansion of newform 3630.2.a.i, downloaded from the LMFDB on 12 May 2024.

# We generate the q-expansion using the Hecke eigenvalues a_p at the primes.
# Each a_p is given as a linear combination
# of the following basis for the coefficient ring.

def make_data():

    from sage.all import prod, floor, prime_powers, gcd, QQ, primes_first_n, next_prime, RR

    def discrete_log(elts, gens, mod):
        # algorithm 2.2, page 16 of https://arxiv.org/abs/0903.2785
        def table_gens(gens, mod):
            T = [1]
            n = len(gens)
            r = [None]*n
            s = [None]*n
            for i in range(n):
                beta = gens[i]
                r[i] = 1
                N = len(T)
                while beta not in T:
                    for Tj in T[:N]:
                        T.append((beta*Tj) % mod)
                    beta = (beta*gens[i]) % mod
                    r[i] += 1
                s[i] = T.index(beta)
            return T, r, s
        T, r, s = table_gens(gens, mod)
        n = len(gens)
        N = [ prod(r[:j]) for j in range(n) ]
        Z = lambda s: [ (floor(s/N[j]) % r[j]) for j in range(n)]
        return [Z(T.index(elt % mod)) for elt in elts]
    def extend_multiplicatively(an):
        for pp in prime_powers(len(an)-1):
            for k in range(1, (len(an) - 1)//pp + 1):
                if gcd(k, pp) == 1:
                    an[pp*k] = an[pp]*an[k]
    from sage.all import PolynomialRing, NumberField
    R = PolynomialRing(QQ, "x")
    f = R(poly_data)
    K = NumberField(f, "a")
    betas = [K.gens()[0]**i for i in range(len(poly_data))]
    convert_elt_to_field = lambda elt: sum(c*beta for c, beta in zip(elt, betas))
    # convert aps to K elements
    primes = primes_first_n(len(aps_data))
    good_primes = [p for p in primes if not p.divides(level)]
    aps = map(convert_elt_to_field, aps_data)
    if not hecke_ring_character_values:
        # trivial character
        char_values = dict(zip(good_primes, [1]*len(good_primes)))
    else:
        gens = [elt[0] for elt in hecke_ring_character_values]
        gens_values = [convert_elt_to_field(elt[1]) for elt in hecke_ring_character_values]
        char_values = dict([(
            p,prod(g**k for g, k in zip(gens_values, elt)))
            for p, elt in zip(good_primes, discrete_log(good_primes, gens, level))
            ])
    an_list_bound = next_prime(primes[-1])
    an = [0]*an_list_bound
    an[1] = 1
    
    from sage.all import PowerSeriesRing
    PS = PowerSeriesRing(K, "q")
    for p, ap in zip(primes, aps):
        if p.divides(level):
            euler_factor = [1, -ap]
        else:
            euler_factor = [1, -ap, p**(weight - 1) * char_values[p]]
        k = RR(an_list_bound).log(p).floor() + 1
        foo = (1/PS(euler_factor)).padded_list(k)
        for i in range(1, k):
            an[p**i] = foo[i]
    extend_multiplicatively(an)
    return PS(an)
level = 3630
weight = 2
poly_data = [0, 1]

# The basis for the coefficient ring is just the power basis
# in the root of the defining polynomial above.
hecke_ring_character_values = None
aps_data = [[-1], [1], [-1], [2], [0], [4], [6], [-2], [4], [2], [-8], [10], [-2], [6], [-8], [-10], [0], [12], [-4], [-8], [8], [6], [-8], [-6], [10], [10], [8], [-16], [16], [6], [-14], [0], [-2], [-2], [14], [6], [18], [12], [8], [-18], [12], [6], [16], [-12], [-18], [-16], [14], [-16], [24], [-22], [26], [-20], [12], [20], [2], [16], [-10], [10], [-28], [-18], [14], [-14], [2], [20], [-10], [18], [-12], [32], [-32], [-20], [18], [-24], [-8], [16], [12], [36], [-22], [-34], [6], [-12], [4], [26], [28], [34], [34], [-32], [18], [-32], [-2], [24], [-32], [36], [-32], [-12], [-4], [24], [-2], [6], [30], [-16], [14], [-14], [12], [42], [-46], [-18], [16], [-22], [-20], [-8], [-38], [32], [30], [-12], [16], [42], [-36], [12], [-10], [-40], [10], [36], [-18], [-36], [-4], [6], [-38], [-24], [-8], [4], [26], [-4], [-32], [-54], [-22], [8], [-42], [-42], [42], [-6], [-10], [-22], [-40], [-48], [-2], [-4], [16], [-10], [-20], [28], [8], [46], [-20], [44], [-12], [16], [-46], [14], [-52], [-46], [-28], [-10], [34], [-32], [-18], [-36], [-16], [-48], [12], [6], [-24], [-10], [24], [-8], [-64], [6], [58], [-26], [-46], [44], [-48], [0], [26], [-26], [-56], [-66], [-52], [-36], [-20], [-36], [14], [-24], [20], [-30], [-28], [42], [32], [30], [30], [-20], [30], [46], [54], [4], [-12], [26], [8], [4], [2], [-36], [36], [-2], [16], [-8], [32], [62], [-18], [-42], [0], [-50], [8], [62], [14], [-64], [12], [4], [-34], [36], [-10], [68], [22], [46], [-18], [-30], [20], [-28], [30], [-6], [60], [-36], [24], [-14], [-24], [-46], [18], [16], [-32], [44], [26], [4], [-12], [30], [64], [38], [74], [36], [-42], [74], [2], [20], [38], [20], [-64], [56], [-78], [28], [-26], [-2], [34], [22], [-58], [-44], [38], [10], [-16], [0], [28], [-68], [32], [8], [32], [32], [12], [68], [-50], [-76], [-22], [14], [-56], [-50], [26], [-72], [-6], [48], [-40], [-42], [32], [-66], [-44], [-54], [-4], [-6], [-42], [72], [76], [54], [52], [-30], [-60], [72], [16], [-62], [-58], [-20], [48], [-12], [24], [72], [-2], [58], [14], [86], [22], [8], [18], [-46], [-4], [4], [24], [-74], [-32], [-6], [-42], [48], [74], [48], [54], [-54], [90], [-54], [-10], [-62], [30], [-16], [-10], [60], [38], [-60], [0], [54], [14], [-6], [-18], [-22], [-68], [30], [24], [-60], [30], [-72], [-64], [22], [-84], [-60], [-60], [-26], [90], [-70], [-16], [-24], [-28], [36], [34], [-86], [-62], [-52], [-76], [-20], [-62], [80], [-78], [34], [2], [30], [58], [-56], [40], [-2], [-66], [0], [34], [46], [-28], [-12], [60], [-40], [-74], [50], [4], [-14], [-20], [-66], [-22], [98], [38], [86], [-10], [54], [52], [12], [-4], [10], [-76], [98], [32], [2], [-76], [-64], [82], [48], [14], [-24], [48], [12], [26], [-18], [12], [-50], [-92], [-60]]
